#include "ofApp.h"

using namespace ofxCv;

//--------------------------------------------------------------
void ofApp::setup(){
    
    //デバイスの向きを縦に固定
    ofSetOrientation(OFXIOS_ORIENTATION_PORTRAIT);

    //インカメラはDeviceID(1)
    cam.setDeviceID(1);
    cam.setup(640, 480);
    
    //コミック風画像のメモリ領域確保
    comic.allocate(cam.getWidth(), cam.getHeight(), OF_IMAGE_GRAYSCALE);
//カスタム色画像のメモリ領域確保
customColor.allocate(cam.getWidth(), cam.getHeight(), GL_RGB);
//カスタム色画像のピクセル（RGBの3色なので、幅x高さx3となる）
pix = new unsigned char[ (int)( cam.getWidth() * cam.getHeight() * 3.0) ];
}

//--------------------------------------------------------------
void ofApp::update(){
    
    cam.update();   //カメラの映像を更新
    
    //カメラ映像が更新されたら
    if(cam.isFrameNew()) {

        //openCVの機能を使ってエフェクトをかける
        //Canny()の引数については以下のリンクを参照のこと
        //http://labs.eecs.tottori-u.ac.jp/sd/Member/oyamada/OpenCV/html/py_tutorials/py_imgproc/py_canny/py_canny.html
        
        ofxCv::convertColor(cam, gray, CV_RGB2GRAY);
        ofxCv::GaussianBlur(gray, blur, 5);
        ofxCv::Canny(blur, edge, 20, 100);
        ofxCv::threshold(cam, thresh, 100.0, false);
        ofxCv::erode(cam, eroded, 5);
        ofxCv::dilate(cam, dilated, 5);

        gray.update();
        blur.update();
        edge.update();
        thresh.update();
        eroded.update();
        dilated.update();

        //comic////////////////////////////
        //blurとedgeを組み合わせてcomicにする
        unsigned char *blurData = blur.getPixels().getData();
        unsigned char *edgeData = edge.getPixels().getData();
        unsigned char *comicData = comic.getPixels().getData();

        for (int i = 0; i < cam.getWidth() * cam.getHeight(); i++) {
            if (edgeData[i] == 0)
                comicData[i] = (blurData[i] / 64) * 64 + 63;
            else
                comicData[i] = 0;
        }
        comic.update();

        //customColor//////////////////////////
        //カスタム色画像
        unsigned char * pixels = cam.getPixels().getData();    //カメラ画像データ
        
        int camW = cam.getWidth();    //撮影画像の幅、高さ
        int camH = cam.getHeight();
        
        //1ピクセルずつ取り出して、r, g, bを抽出
        for (int y = 0; y < camH; y ++){
            for (int x = 0; x < camW; x ++){
                
                float r = pixels[y*camW*3 + x*3];    //赤を抽出
                float g = pixels[y*camW*3 + x*3 + 1];    //緑を抽出
                float b = pixels[y*camW*3 + x*3 + 2];    //青を抽出
        
                //明るさ、コントラスト//////////////////////
                //brightness = 0, contrast = 1.0で通常の映像
                /*
                float brightness = 50;
                float contrast = 0.5;

                r = contrast*(r - 128) + 128 + brightness;
                g = contrast*(g - 128) + 128 + brightness;
                b = contrast*(b - 128) + 128 + brightness;
                */
                //色の反転//////////////////////
                /*
                r = 255 - r;
                g = 255 - g;
                b = 255 - b;
                */
                //セピア//////////////////////
                /*
                r = (int)((float)r/255*240);
                g = (int)((float)g/255*200);
                b = (int)((float)b/255*145);
                */
                //2色//////////////////////
                /*
                int brightness = (r + g + b)/3;

                if(brightness < 127){

                r = 32;     //赤
                g = 225;     //緑
                b = 0;     //青

                }else if(brightness >=127){

                r = 255;     //赤
                g = 180;     //緑
                b = 0;     //青
                }
                */
                //3色//////////////////////
                /*
                int brightness = (r + g + b)/3;

                if(brightness < 85){
                    r = 0;     //赤
                    g = 50;     //緑
                    b = 0;     //青

                }else if(brightness >=85 && brightness < 170){
                    r = 0;     //赤
                    g = 150;     //緑
                    b = 255;     //青

                }else if(brightness >= 170){
                    r = 255;     //赤
                    g = 100;     //緑
                    b = 0;     //青
                }
                */
                
                //変換後の値をpixに保存
                pix[y*camW*3 + x*3] = ofClamp(r, 0, 255);   //赤
                pix[y*camW*3 + x*3 + 1] = ofClamp(g, 0, 255);   //緑
                pix[y*camW*3 + x*3 + 2] = ofClamp(b, 0, 255);;   //青
            }
        }
        //customColorを更新
        customColor.loadData(pix, cam.getWidth(), cam.getHeight(), GL_RGB);
    }
}

//--------------------------------------------------------------
void ofApp::draw(){
    ofBackground(255, 255, 255);

    ofPushMatrix();
    ofSetColor(255);

    //画面を鏡面にする
    ofTranslate(ofGetWidth(), 0);
    ofScale(-1, 1);
    //キャプチャ映像を画面に合わせて拡大
    float scale = ofGetWidth() / cam.getWidth();
    ofScale(scale, scale);
    
    //ここの〇〇.draw()をそれぞれ実行してみる
    if(cam.isFrameNew()) {
        //cam.draw(0, 0);
        //edge.draw(0, 0);
        //comic.draw(0, 0);
        //thresh.draw(0, 0);
        //eroded.draw(0, 0);
        //dilated.draw(0, 0);
        customColor.draw(0, 0);
    }

    ofPopMatrix();
}

//--------------------------------------------------------------
void ofApp::exit(){

}

//--------------------------------------------------------------
void ofApp::touchDown(ofTouchEventArgs & touch){

}

//--------------------------------------------------------------
void ofApp::touchMoved(ofTouchEventArgs & touch){

}

//--------------------------------------------------------------
void ofApp::touchUp(ofTouchEventArgs & touch){

}

//--------------------------------------------------------------
void ofApp::touchDoubleTap(ofTouchEventArgs & touch){
 
}

//--------------------------------------------------------------
void ofApp::touchCancelled(ofTouchEventArgs & touch){
    
}

//--------------------------------------------------------------
void ofApp::lostFocus(){

}

//--------------------------------------------------------------
void ofApp::gotFocus(){

}

//--------------------------------------------------------------
void ofApp::gotMemoryWarning(){

}

//--------------------------------------------------------------
void ofApp::deviceOrientationChanged(int newOrientation){

}

